/******************************************************************************
* treeMenuCode.js                                                             *
*                                                                             *
* Copyright 1999 by Mike Hall.                                                *
* Web address: http://www.brainjar.com                                        *
* Last update: September 17, 1999                                             *
*                                                                             *
* Creates an expanding and collapsing menu with files and folders in a child  *
* frame.                                                                      *
*******************************************************************************
* Adapted by Jaap Reitsma for Telematica Instituut nov 1999.                  *
* Adapted by Harm Bakker for BiZZdesign jan-march 2002.                       *
* Lots of adaptations by FransF and PeterZ, BiZZdesign, 2002                  *
******************************************************************************/

///////////////////////////////////////////////////////////////////////////////
// Note: Please write as little style information as possible with this code,
// and do your formatting in the ReportStyle.css stylesheet. This enables
// users to override things with a custom css.
///////////////////////////////////////////////////////////////////////////////


/////////////////////////////////////////////////////
// Global tree store and methods
/////////////////////////////////////////////////////

var _trees = new Array();

function createTree(name, text, url, icon, flags)
{
    if (name != null && name != '')
    {
        if (getTree(name) != null)
        {
            alert('Tree \'' + name + '\'already exists!');
            return null;
        }
        var tree = new Tree(name, text, url, icon);
        _trees[_trees.length] = tree;

        if (getTree(name) == null)
            alert('Failed to create tree \'' + name + '\'!');

        if (flags != null && flags != '')
            tree.setFlags(flags);

        return tree;
    }
    return null;
}

function getTree(name, error)
{
    for (var idx = 0; idx < _trees.length; idx++)
    {
        if (_trees[idx].name == name)
            return _trees[idx];
    }
    if (error)
        alert('Tree \'' + name + '\' not found!');
    return null;
}


/////////////////////////////////////////////////////
// Tree object definitions
/////////////////////////////////////////////////////

// Constructor.
function TreeItem(tree, text, url, icon, expandedText, collapsedText, trailingIcon, trailingUrl, trailingTooltip)
{
    this.tree = tree;               // the tree this item is part of
    this.text = (text != null ? text : '');
    this.url = (url != null ? url : '');
    this.icon = (icon != null ? icon : '');
    this.expandedText = (expandedText != null ? expandedText : '');     // text to append when item in expanded state
    this.collapsedText = (collapsedText != null ? collapsedText : '');  // text to append when item in collapsed state
    this.trailingIcon = (trailingIcon != null ? trailingIcon : '');
    this.trailingUrl = (trailingUrl != null ? trailingUrl : '');
    this.trailingTooltip = (trailingTooltip != null ? trailingTooltip : '');
    this.subtree = null;
    this.index = tree.itemIndex++;
    this.createSubtree = treeItemCreateSubtree;
    this.isEmpty = treeItemIsEmpty;
    return;
}
// Methods.
function treeItemCreateSubtree()
{
    this.subtree = new Subtree(this.tree);
    return this.subtree;
}
function treeItemIsEmpty()
{
    if ((this.text == '') && (this.url == '') && (this.icon == ''))
        return true;
    else
        return false;
}

// Constructor.
function Subtree(tree)
{
    this.tree = tree;               // the tree this subtree is part of
    this.items = new Array();
    this.addItem = subtreeAddItem;
    return;
}
// Methods.
function subtreeAddItem(text, url, icon, expandedText, collapsedText, trailingIcon, trailingUrl, trailingTooltip)
{
    var item = new TreeItem(this.tree, text, url, icon, expandedText, collapsedText, trailingIcon, trailingUrl, trailingTooltip);
    this.items[this.items.length] = item;
    return item;
}

// Constructor
function Tree(name, text, url, icon)
{
    this.name = name;
    this.itemIndex = 0;                 // item counter (init before first new item!)
    this.root = new TreeItem(this, text, url, icon);

    // Volatile members
    this.itemIsExpanded = null;         // item on this row is expanded
    this.selectedItem = 0;              // the row index of the selected item
    this.selectedItemFound = false;     // flag 'selected item seen', for scrolling purposes
    this.scrollOffsetStart = 0;         // offset of first item of interest (that must be visible)
    this.scrollOffsetEnd = 0;           // offset of last item of iterest (that should be visible)
    this.inSubtreeSelectedItem = false; // flag that says if we are displaying an item of interest
    this.someItemWasExpanded = false;
    this.currentDepth = 0;              // current menu level
    this.itemHasLine = null;            // array; keeps track of tree lines that cross this item's row 
    
    this.document = null;               // the document this tree is on
    this.callerWindow = null;           // window containing the document: only necessary for scroll using layers

    // Fixed members
    this.showPlusMinus = true;
    this.showToolTip = true;
    this.showIcons = true;
    this.showLines = true;
    this.imageFolder = './graphics/';
    this.scrollItemHeight = 18;         // vertical scroll increment per tree item in pixels
    this.scrollPadding = 20;		// padding between area of interest and frame edges (for scrollbar, title bar)

    this.initialize = treeInitialize;
    this.setFlags = treeSetFlags;
}
// Methods
function treeInitialize()
{
    // One-time initialization, could not be done in constructor
    if (this.itemIsExpanded == null)
    {
        this.itemIsExpanded = new Array();
        for (var i = 0; i < this.itemIndex; i++)
            this.itemIsExpanded[i] = false;
        this.selectedItem = 0;
    }
    this.selectedItemFound = false;
    this.inSubtreeSelectedItem = false;
    this.scrollOffsetStart = 0;
    this.scrollOffsetEnd = this.scrollOffsetStart + this.scrollItemHeight;
    this.currentDepth = 0;
    this.itemHasLine = new Array();
}
function treeSetFlags(flags)
{
    if (flags == null || flags == '')
        return;
    var allflags = flags.split(" ");
    for (var idx = 0; idx < allflags.length; idx++)
    {
        var flag = allflags[idx].split('=');
        if (flag[0] == 'plusminus') this.showPlusMinus = (flag[1] == 'yes');
        else if (flag[0] == 'tooltip') this.showToolTip = (flag[1] == 'yes');
        else if (flag[0] == 'icons') this.showIcons = (flag[1] == 'yes');
        else if (flag[0] == 'lines') this.showLines = (flag[1] == 'yes');
    }
}


/////////////////////////////////////////////////////
// Global tree functionality
/////////////////////////////////////////////////////

function treeMenuDisplay(treeName, callerWindow)
{
    var tree = getTree(treeName, true);
    if (tree == null)
        return;
    // When this function is called without specifying the caller,
    // use the document stored in the tree
    if (callerWindow != null)
    {
        tree.callerWindow = callerWindow;
        tree.document = callerWindow.document;
    }
    tree.initialize();

    // tree.document not always available immediately, defer use
    setTimeout('buildAndReplaceTreeContents(\'' + treeName + '\')', 10);

    return;
}

function buildAndReplaceTreeContents(treeName)
{
    var tree = getTree(treeName, true);
    if (tree == null)
        return;
    if (tree.callerWindow == null ||    // there must be a window for this tree,...
        tree.callerWindow.closed ||     // ...which is not closed,
        tree.document == null ||        // ...and has a document loaded,
        tree.document.body == null)     // ...which has a body to put the tree in
        return;

    tree.document.body.onLoad = 'handleTitleBar(); //treeMenuScroll(\'' + treeName + '\');';
    // Note: most style elements are placed in a css,
    //       but cellspacing must be specified in the element itself
    var sTreeHtml = '<TABLE class="' + treeName + '" cellspacing="0">';
    if (!tree.root.isEmpty())
        sTreeHtml += treeMenuDisplayItem(tree, tree.root, true, false, true);
    tree.currentDepth++;
    sTreeHtml += treeMenuListItems(tree, tree.root.subtree, tree.root.isEmpty());
    tree.currentDepth--;
    sTreeHtml += '</TABLE>';

    tree.document.all.framecontent.innerHTML = sTreeHtml;

    treeMenuScroll(treeName);
}

//----------------------------------------------------------------------------
// This function displays each item in the given menu or submenu.
//----------------------------------------------------------------------------

function treeMenuListItems(tree, branch, first_is_top)
{
    var first_of_all, is_last, is_expanded, out = '';

    for (var i = 0; i < branch.items.length; i++)
    {
        first_of_all = (i == 0 && first_is_top);
        is_last = (i == branch.items.length - 1);
        is_expanded = tree.itemIsExpanded[branch.items[i].index];

        var selectedItemFoundBefore = tree.selectedItemFound;

        out += treeMenuDisplayItem(tree, branch.items[i], first_of_all, is_last, is_expanded);
        // If the item is a submenu and it is expanded, make a recursive call to
        // draw its item list.
        if (branch.items[i].subtree &&  is_expanded)
        {
            if (!selectedItemFoundBefore && tree.selectedItemFound)
            {
                tree.inSubtreeSelectedItem = true;
            }
            tree.itemHasLine[tree.currentDepth] = (!is_last);
            tree.currentDepth++;
            out += treeMenuListItems(tree, branch.items[i].subtree, false);
            tree.currentDepth--;
            if (!selectedItemFoundBefore && tree.selectedItemFound)
            {
                tree.inSubtreeSelectedItem = false;
            }
        }
        else
        {
            tree.itemHasLine[tree.currentDepth] = false;
        }
    }
    return out;
}

//----------------------------------------------------------------------------
// This displays a single menu or submenu item.
//----------------------------------------------------------------------------

function treeMenuDisplayItem(tree, item, isFirstOfAll, isLast, isExpanded)
{
    var out = '', i;

    // Update vertical scrolling amount until we find the selected item.
    if (item.index == tree.selectedItem)
        tree.selectedItemFound = true;

    if (!tree.selectedItemFound)
    {
        tree.scrollOffsetStart += tree.scrollItemHeight;
        tree.scrollOffsetEnd =  tree.scrollOffsetStart + tree.scrollItemHeight;
    }
    else if (tree.inSubtreeSelectedItem && tree.someItemWasExpanded)
    {
        tree.scrollOffsetEnd += tree.scrollItemHeight;
    }

    // Start the table row.
    out = '<TR class="' + tree.name + '"><TD class="' + tree.name + '">';

    if (tree.currentDepth > 0)
    {
        // Draw descending bars from upper levels, also set horizontal scrolling
        // amount if this is the selected item.
        for (i = 1; i < tree.currentDepth; i++)
        {
            out += '<IMG class="' + tree.name + '" src="' + tree.imageFolder;
            out += (tree.showLines && tree.itemHasLine[i] ? 'tree_bar.gif' : 'tree_spacer.gif');
            out += '" align="left" border="0" vspace="0" hspace="0">';
//             if (item.index == tree.selectedItem)
//                 tree.scrollRight();
        }
        // Determine whether we want to show the plus/minus icons
        if (tree.showPlusMinus &&
            item.subtree != null
            && item.subtree.items.length > 0)
        {
            // Make the plus/minus clickable, so that it expands/collapses without
            // changing the selection
            out += '<IMG class="' + tree.name + '" src="' + tree.imageFolder;
            if (tree.showLines)
            {
                if (isExpanded)
                    out += (isLast ? 'tree_corner_up_minus.gif' : (isFirstOfAll ? 'tree_corner_down_minus.gif' : 'tree_tee_minus.gif'));
                else
                    out += (isLast ? 'tree_corner_up_plus.gif' : (isFirstOfAll ? 'tree_corner_down_plus.gif' : 'tree_tee_plus.gif'));
            }
            else
            {
                if (isExpanded)
                    out += 'tree_minus.gif';
                else
                    out += 'tree_plus.gif';
            }
            out += '" align="left" border="0" vspace="0" hspace="0"';
            out += ' onClick="javascript: return treeItemClick(\'';
            out += tree.name + '\', ';
            out += item.index + ', false, true);">';
        }
        else
        {
            // Right. No plus nor minus, instead, show a tee or a corner
            out += '<IMG class="' + tree.name + '" src="' + tree.imageFolder;
            if (tree.showLines)
                out += (isLast ? 'tree_corner_up.gif' : (isFirstOfAll ? 'tree_corner_down.gif' : 'tree_tee.gif'));
            else
                out += 'tree_spacer.gif';
            out += '" align="left" border="0" vspace="0" hspace="0">';
        }
    }

    out += '<SPAN class="' + tree.name + '"';
    if (item.url != '')
    {
        out += '" onmouseover="javascript:this.className=\'' + tree.name + '-Item-Hover\';"';
        out += '" onmouseout="javascript:this.className=\'' + tree.name + '-Item\';"';
    }
    // If the item is collapsed, clicking it also expands it
//    if (!isExpanded)
//    {
//        out += '" onClick="javascript:';
//        out += (item.url == '' ? '' : item.url + '; ');
//        out += 'return treeItemClick(\'';
//        out += tree.name + '\', ';
//        out += item.index + ', ' + (item.url != '') + ', ' + (item.subtree != null) + ');"';
//    }
//    else
//    {
        if (item.url != '')
        {
            out += '" onClick="';
            out += (item.url == '' ? '' : item.url + ';"');
        }
//    }
    out += '>';

    // Add a placeholder for the icon image. The image is
    // later inserted by manipulating the background using the css
    if (tree.showIcons)
    {
        out += '<SPAN class="' + tree.name + '-Image-';
        if (item.icon != '')
            out += item.icon;
        else if (item.subtree)
            out += (isExpanded ? (tree.showLines ? 'folderopen' : 'folderopennoline') : 'folderclosed');
        else if (item.url.indexOf('http://') == 0)
            out += 'linkexternal';
        else
            out += 'linklocal';
        out += (tree.showToolTip ? '" TITLE="' + item.text : '') + '"></SPAN>';
    }

    // Write the link text and finish the link and table row.
    out += '<SPAN class="' + tree.name + '-level' + tree.currentDepth + '">';
    if (item.index == tree.selectedItem)
    {
        out += '<SPAN class="' + tree.name + '-Selected">';
        out += item.text;
        out += '</SPAN>';
    }
    else
    {
        out += item.text;
    }
    // Append extra label text
    if (isExpanded)
    {
        if (item.expandedText != null && item.expandedText != '')
        {
            out += item.expandedText;
        }
    }
    else
    {
        if (item.collapsedText != null && item.collapsedText != '')
        {
            out += item.collapsedText;
        }
    }
    out += '</SPAN></SPAN>';

    // Add a placeholder for the clickable trailing icon image. The image is
    // later inserted by manipulating the background using the css
    if (tree.showIcons && item.trailingIcon != '')
    {
        out += '&nbsp;&nbsp;<SPAN class="' + tree.name + '-Image-';
        out += item.trailingIcon;
        out += (tree.showToolTip ? '" TITLE="' + item.trailingTooltip : '') + '"';
        if (item.trailingUrl != '')
        {
            out += ' onClick="' + item.trailingUrl + '"';
        }
        out += '></SPAN>';
    }

    out += '</TD></TR>';

    return out;
}

//----------------------------------------------------------------------------
// This function handles a click on a menu item.
//----------------------------------------------------------------------------

function treeItemClick(treeName, n, link, toggleExpand)
{
    var update_it = false;

    var tree = getTree(treeName, true);
    if (tree == null)
        return;

    // If this is a submenu, toggle the expansion flag.
    if (toggleExpand)
    {
        tree.itemIsExpanded[n] = (!tree.itemIsExpanded[n]);
        if (tree.itemIsExpanded[n])
            tree.someItemWasExpanded = true;
        update_it = true;
    }

//    if (link)
//    {
        if (tree.selectedItem != n)
            update_it = true;
        // Save the selected item index
        tree.selectedItem = n;
//    }

    // Set up redraw the menu frame.
    // The link needs updating to highlight the selection
    if (update_it)
        setTimeout('treeMenuDisplay(\'' + treeName + '\')', 10);

    return link;
}

function treeMenuScroll(treeName)
{
    var frameTop, frameLeft, frameHeight, frameWidth;
    var tree = getTree(treeName, true);
    if (tree == null)
        return;
    if (!tree.selectedItemFound)
        return;

    tree.someItemWasExpanded = false;    // reset for next display

    var wind = tree.callerWindow;
    if (wind == null)
        wind = window;

    frameTop = tree.document.body.scrollTop + tree.scrollPadding;
//    frameLeft = tree.document.body.scrollLeft;
    frameHeight = tree.document.body.clientHeight - 2 * tree.scrollPadding;
//    frameWidth = tree.document.body.clientWidth;

    // Scroll the frame if necessary.
    var scrollHeight = tree.scrollOffsetEnd - tree.scrollOffsetStart;
    if (scrollHeight <= frameHeight)
    {
        if (tree.scrollOffsetStart < frameTop)
        {
            wind.scrollTo(0, tree.scrollOffsetStart);
        }
        else if (tree.scrollOffsetEnd > (frameTop + frameHeight))
        {
            wind.scrollTo(0, frameTop - frameHeight + tree.scrollOffsetEnd);
        }
    }
    else
    {
        wind.scrollTo(0, tree.scrollOffsetStart);
    }
    return;
}

//----------------------------------------------------------------------------
// Returns the index of the tree item with the specified URL. If
// the item is found in a collapsed branch, it is expanded.
//----------------------------------------------------------------------------
function locateItemWithUrl(tree, item, url)
{
    if (item.url.indexOf(url) >= 0)
        return item.index;

    var subtree = item.subtree;
    if (subtree != null)
    {
        for (var i = 0; i < subtree.items.length; i++)
        {
            var selIndex = locateItemWithUrl(tree, subtree.items[i], url);
            if (selIndex >= 0)
            {
                if (!tree.itemIsExpanded[item.index])
                {
                    tree.itemIsExpanded[item.index] = true;
                    tree.someItemWasExpanded = true;
                }
                return selIndex;
            }
        }
    }
    return -1;  
}

//----------------------------------------------------------------------------
// Selects the tree item with the specified URL. Expands a branch if necessary.
//----------------------------------------------------------------------------
function selectItemOnURL(treeName, url, expand)
{
    var tree = getTree(treeName, true);
    if (tree == null)
        return;

    if (expand == null)
    expand = false;

    // Extract filename + extension + hash part
    url = url.substring(url.lastIndexOf('/') + 1, url.length);

    selIndex = locateItemWithUrl(tree, tree.root, url);

    if (selIndex >= 0)
    {   
        if (tree.selectedItem != selIndex)   // if already selected, do nothing
        {
            tree.selectedItem = selIndex;
            if (expand)
            {
                if (!tree.itemIsExpanded[selIndex])
                {
                    tree.itemIsExpanded[selIndex] = true;
                    tree.someItemWasExpanded = true;
                }
            }
            setTimeout('treeMenuDisplay(\'' + treeName + '\')', 10);
        }
    }
}

function locateItemWithLabel(tree, item, label)
{
    if (item.text.length == label.length &&
        item.text.indexOf(label) >= 0)
        return item.index;

    var subtree = item.subtree;
    if (subtree != null)
    {
        for (var i = 0; i < subtree.items.length; i++)
        {
            var selIndex = locateItemWithLabel(tree, subtree.items[i], label);
            if (selIndex >= 0)
            {
                if (!tree.itemIsExpanded[item.index])
                {
                    tree.itemIsExpanded[item.index] = true;
                    tree.someItemWasExpanded = true;
                }
                return selIndex;
            }
        }
    }
    return -1;  
}

function selectItemOnLabel(treeName, label, expand)
{
    var tree = getTree(treeName, true);
    if (tree == null)
        return;
    
    if (expand == null)
        expand = false;

    selIndex = locateItemWithLabel(tree, tree.root, label);

    if (selIndex >= 0)
    {   
        if (tree.selectedItem != selIndex)   // if already selected, do nothing
        {
            tree.selectedItem = selIndex;
            if (expand)
            {
                if (!tree.itemIsExpanded[selIndex])
                {
                    tree.itemIsExpanded[selIndex] = true;
                    tree.someItemWasExpanded = true;
                }
            }
            setTimeout('treeMenuDisplay(\'' + treeName + '\')', 10);
        }
    }
}
